@@ -1,5 +1,5 @@
 package v6;
-$v6::VERSION = '0.015';
+$v6::VERSION = '0.016';
 
 # Documentation in the __END__
 use 5.006;
@@ -9,6 +9,9 @@ use Module::Compile-base;
 use File::Basename;
 # use Pugs::Runtime::Perl6;
 
+my $backend_identifier = 'perl5';
+my $grammar_identifier;
+
 my $bin;
 BEGIN { $bin = ((dirname(__FILE__) || '.') . "/..") };
 
@@ -21,6 +24,22 @@ INIT {
   }
 }
 
+sub banner { 
+q(         ___ 
+        /  __\
+ __    _\ \___      _____   _ ___ ___
+/\ \  / /\  __ \   /\  __ \/\  __  __ \
+\ \ \/ /\ \ \_\ \__\ \ \_\ \ \ \/\ \/\ \
+ \ \__/  \ \____/\_).q(\\).q(\ \  __/\ \_\ \_\ \_\  Perl6-in-Perl5
+  \/_/    \ ___/\/_/ \ \ \/  \/_/\/_/\/_/  
+                      \ \_\              Version: ) . $v6::VERSION . q(
+                       \/_/   Copyright 2006, The Pugs Contributors
+--------------------------------------------------------------------
+ Web: http://pugscode.org/           Email: perl6-compiler@perl.org
+
+);
+}
+
 sub pmc_can_output { 1 }
 
 sub pmc_parse_blocks {
@@ -41,7 +60,7 @@ sub pmc_compile {
 
     require Pugs::Compiler::Perl6;
 
-    my $p6 = Pugs::Compiler::Perl6->compile( $source );
+    my $p6 = Pugs::Compiler::Perl6->compile( $source, { backend => $backend_identifier, grammar => $grammar_identifier } );
     my $perl5 = $p6->{perl5};
 
     # Don't write when we failed to compile, otherwise it never recompiles!
@@ -50,10 +69,20 @@ sub pmc_compile {
 
     # $perl5 =~ s/do\{(.*)\}/$1/s;
     my ($package, $file) = caller(4);
+
+    # allow 'regex' declarations:
+    #     use base 'Pugs::Grammar::Base';  
+    #   see: t/rules/from_perl6_rules/capture.t
+
+    # postprocess perl5 backend
+    if (  $backend_identifier eq 'perl5'
+       || $backend_identifier eq 'perl5:Pugs::Emitter::Perl6::Perl5'
+       ) {
+
     $perl5 = 
         ( $package 
             ? "package $package;\n" 
-            : "package main; # no package name\n" ).
+            : "package Main; # no package name\n" ).
         ((!$package or ($package eq 'main')) ? (
             "use Config;\n".
             "use lib split(/\\Q\$Config{path_sep}/, \$ENV{PERL6LIB} || '');\n"
@@ -61,12 +90,15 @@ sub pmc_compile {
         "use Scalar::Util;
          use Pugs::Runtime::Perl6;
          use Pugs::Runtime::Perl6Prelude;
+         use Pugs::Runtime::Perl5Container;
+         use base 'Pugs::Grammar::Base';  
          use strict;
          no strict 'refs';
          no warnings ('void', 'uninitialized');
          \$::_V6_COMPILER_OS      = '$^O';
          \$::_V6_COMPILER_NAME    = 'v6.pm';
          \$::_V6_COMPILER_VERSION = '$v6::VERSION';
+         undef \$::_V6_MATCH_;
          my \%_V6_PAD;
          our \%_V6_STATE;
         " .  
@@ -74,7 +106,7 @@ sub pmc_compile {
         $perl5 . "\n" .
         "; 1;\n";
 
-    unless ( $ENV{V6NOTIDY} )
+    if ( $ENV{V6TIDY} )
     {
       # Perl::Tidy is used if available
       local $@;   # don't care if there are errors here
@@ -94,9 +126,13 @@ sub pmc_compile {
       }
     }
 
+    }  # / postprocess perl5 backend
+
     return $perl5;
 }
 
+#print "ARGS: \$0='$0' \@ARGV=[ @ARGV ]\n";
+
 if (@ARGV and !caller) {
     # We are the main program here
     my ($compile_only, $code);
@@ -107,15 +143,33 @@ if (@ARGV and !caller) {
     }
 
     shift(@ARGV) if $ARGV[0] =~ /^--pugs/;
-    shift(@ARGV) if $ARGV[0] =~ /^-Bperl5$/i;
+    if ( $ARGV[0] =~ /^-B(.*)/i ) {
+        shift(@ARGV);
+        $backend_identifier = $1 if $1;
+    }
     splice(@ARGV, 0, 2) if $ARGV[0] =~ /^-B$/;
 
+    if ( $ARGV[0] =~ /^-G(.*)/i ) {
+        shift(@ARGV);
+        $grammar_identifier = $1 if $1;
+    }
+
     while (@ARGV and $ARGV[0] =~ /^-(\w)(.+)/) {
         use Config;
-        $ENV{PERL6LIB} = "$2$Config{path_sep}$ENV{PERL6LIB}" if $1 eq 'I';
+        if($1 eq 'I') {
+            $ENV{PERL6LIB} = ((defined($ENV{PERL6LIB}) && $ENV{PERL6LIB} ne '')
+                              ? "$Config{path_sep}$ENV{PERL6LIB}"
+                              : "");
+            $ENV{PERL6LIB} = "$2$ENV{PERL6LIB}";
+        }
         shift @ARGV;
     }
 
+    if (@ARGV && $ARGV[0] eq '-v' ) {
+        print banner();
+        exit 0;
+    }
+
     if (@ARGV and $ARGV[0] =~ s/^-e//) {
         $code = (length($ARGV[0]) ? $ARGV[0] : $ARGV[1]);
     }
@@ -134,6 +188,10 @@ if (@ARGV and !caller) {
         exit 0;
     }
 }
+elsif ( $0 eq '-e' && $ARGV[-1] eq '-v' ) {
+    print banner();
+    exit 0;
+}
 elsif ( $0 eq '-e' ) {
     # perl -Ilib -e 'use v6-alpha' ' "hello world".say '
     # perl -Ilib -e 'use v6-alpha' - ' "hello world".say '
@@ -152,12 +210,25 @@ elsif ( $0 eq '-e' ) {
     }
 
     shift(@ARGV) if $ARGV[0] && $ARGV[0] =~ /^--pugs/;
-    shift(@ARGV) if $ARGV[0] && $ARGV[0] =~ /^-Bperl5$/i;
+    if ( $ARGV[0] =~ /^-B(.*)/i ) {
+        shift(@ARGV);
+        $backend_identifier = $1 if $1;
+    }
     splice(@ARGV, 0, 2) if $ARGV[0] && $ARGV[0] =~ /^-B$/;
 
+    if ( $ARGV[0] =~ /^-G(.*)/i ) {
+        shift(@ARGV);
+        $grammar_identifier = $1 if $1;
+    }
+
     while (@ARGV and $ARGV[0] =~ /^-(\w)(.+)/) {
         use Config;
-        $ENV{PERL6LIB} = "$2$Config{path_sep}$ENV{PERL6LIB}" if $1 eq 'I';
+        if($1 eq 'I') {
+            $ENV{PERL6LIB} = ((defined($ENV{PERL6LIB}) && $ENV{PERL6LIB} ne '')
+                              ? "$Config{path_sep}$ENV{PERL6LIB}"
+                              : "");
+            $ENV{PERL6LIB} = "$2$ENV{PERL6LIB}";
+        }
         shift @ARGV;
     }
 
@@ -210,6 +281,11 @@ This examples assume that v6.pm is in the C<./lib> directory:
     $ perl lib/v6.pm -e 'for 1,2,3 -> $x { say $x }'
     $ perl lib/v6.pm --compile-only -e '<hello>.say;'
 
+Print the version banner:
+
+    $ perl -e 'use v6-alpha' - -v
+    $ perl lib/v6.pm -v
+
 =head1 DESCRIPTION
 
 The C<v6> module is a front-end to the experimental Perl6-to-Perl5 compiler.
@@ -219,14 +295,29 @@ Perl 6 syntax and semantics.
 
 =head2 Other Perl 6 implementations
 
-The Pugs/Haskell Perl 6 is currently the most complete implementation. 
-Pugs currently has some issues with Perl 5 interoperability.
+C<Pugs> Perl 6 is currently the most complete implementation. Pugs is written on top of Haskell.
 
-Parrot Perl 6 aims to become the best performing implementation, but
-currently is less complete than both C<v6.pm> and Pugs/Haskell.
+Perl 6 on C<Parrot> aims to become the best performing implementation.
+Parrot is a virtual machine designed to efficiently compile and execute bytecode for interpreted languages.
 
-Although running C<v6.pm> requires the installation of a lot of Perl 5 modules,
-it is completely independent of Pugs or Parrot. 
+C<v6.pm> is completely independent of Pugs or Parrot. 
+
+=head2 Compile-time system
+
+L<Pugs::Compiler::Rule> provides an implementation for Perl 6 Rules, which are used to define the Perl 6 grammar.
+
+L<Parse::Yapp> is used for implementing the operator precedence parser.
+
+L<Module::Compile> and L<Cache::Cache> provide the precompilation cache
+infrastructure.
+
+=head2 Runtime system
+
+The object system is provided by L<Moose>. 
+Moose is an extension of the Perl 5 object system.
+Moose is built on top of L<Class::MOP>, which is a metaclass system for Perl 5.
+
+L<Data::Bind> and L<Sub::Multi> implement the semantics for perl6-style variable binding, as well as subroutine call argument passing and binding, in Perl 5.
 
 =head1 REQUIREMENTS
 
@@ -278,20 +369,39 @@ Same usage as PERL5LIB - sets the search path for Perl 6 modules.
 
 If set, the compiler will dump the syntax tree to C<STDOUT> just before emitting code, using C<Data::Dumper>.
 
-* V6NOTIDY
+* V6TIDY
 
-If set, the compiler output will be much less readable, but there will
-be some improvement in compiler speed.
+If set, the compiler output will be much more readable, but there will
+be a lot of slowdown in compiler speed.
 
 =head1 COMMAND LINE SWITCHES
 
 * --compile-only
 
-When using v6.pm from the command line, dumps the emitted code
+When using C<v6.pm> from the command line, dumps the emitted code
 to C<STDOUT> and then exit:
 
+    $ perl -e 'use v6-alpha' - --compile-only ' 42.say '
+
     $ perl -Ilib lib/v6.pm --compile-only -e '<hello>.say;'
 
+* -B
+
+Selects alternate code generation backends.
+
+The default is '-Bperl5:Pugs::Emitter::Perl6::Perl5'.
+'-Bperl5' also invokes the default backend.
+
+    $ perl -e 'use v6-alpha' - --compile-only -Bperl5:MyEmitter ' 42.say '
+
+The backend module must provide the C<emit($grammar, $ast)> subroutine.
+
+* -G
+
+Selects alternate grammar frontends.
+
+The default is '-Gperl5:Pugs::Grammar::Perl6'.
+
 =head1 AUTHORS
 
 The Pugs Team E<lt>perl6-compiler@perl.orgE<gt>.
